import * as React from 'react';
import {
  RESOURCE_EDITOR_ID,
  BINARY_DIFF_EDITOR_ID,
  SIDE_BY_SIDE_EDITOR_ID,
  TEXT_DIFF_EDITOR_ID,
} from '../constants';
import {
  MonacoEditor,
  MonacoDiffEditor,
} from '@workbench-stack/components';
import { Registry } from '@vscode/monaco-editor/esm/vs/platform/registry/common/platform';
import { IEditorRegistry, CacheTypenames, IEditorContributionProps } from '@workbench-stack/core';
import { ClientExtensions } from '../constants';
import { EditorContributionFill, IEditorComponentProps } from './editor';
import { Slot, IMonacoEditorProps, IMonacoEditorTextProps, IMonacoDiffEditorProps } from '@workbench-stack/components';
import { URI } from '@vscode/monaco-editor/esm/vs/base/common/uri';


export type IEditorTextProps = IMonacoEditorTextProps  & IEditorComponentProps;
export type IEditorDiffTextProps = IMonacoDiffEditorProps & IEditorComponentProps;

export type IEditorProps = IEditorTextProps | IEditorDiffTextProps;

// String Editor
export const ResourceEditor: React.FC<IEditorContributionProps> = props => {

  return (
    <EditorContributionFill >
      {(editorProps: IEditorTextProps) => {
        const uri = URI.parse(editorProps.input.resource);
        return editorProps.visibleEditorId === props.ID ? (
          <MonacoEditor
            context={editorProps.context}
            defaultValue={editorProps.defaultValue}
            doUpdateValue={editorProps.doUpdateValue}
            enableApi={editorProps.enableApi}
            editorDidMount={editorProps.editorDidMount}
            editorWillMount={editorProps.editorWillMount}
            getEditorState={editorProps.getEditorState}
            theme={editorProps.theme}
            uri={uri}
            height={editorProps.height}
            width={editorProps.width}
            language={editorProps.language}
            onChange={editorProps.onChange}
            options={editorProps.options}
            value={editorProps.value}
            updateEditorState={editorProps.updateEditorState}
            requireConfig={editorProps.requireConfig}
          />
        ) : null;
      }}
    </EditorContributionFill>
  );
};
ResourceEditor.defaultProps = { ID: RESOURCE_EDITOR_ID};
/**
 * Text diff editor id.
 */
export const TextDiffEditor: React.FC<IEditorContributionProps> = props => {
  return (
    <EditorContributionFill>
      {(editorProps: IEditorDiffTextProps) =>
        editorProps.visibleEditorId === props.ID ? (
          <MonacoDiffEditor {...editorProps} />
        ) : null
      }
    </EditorContributionFill>
  );
};
TextDiffEditor.defaultProps = { ID: TEXT_DIFF_EDITOR_ID };


/**
 * Binary diff editor.
 */
export const BinaryResourceDiffEditor: React.FC<IEditorContributionProps> = props => {

  return (
    <EditorContributionFill>
      {(editorProps: IEditorDiffTextProps) =>
        editorProps.visibleEditorId === props.ID ? (
          <MonacoDiffEditor {...editorProps} />
        ) : null
      }
    </EditorContributionFill>
  );
};
BinaryResourceDiffEditor.defaultProps = { ID: BINARY_DIFF_EDITOR_ID };

export const SideBySideEditor: React.FC<IEditorContributionProps> = props => {

    return (
      <EditorContributionFill>
        {(editorProps: IEditorDiffTextProps) =>
          editorProps.visibleEditorId === props.ID ? (
            <MonacoDiffEditor {...editorProps} />
          ) : null
        }
      </EditorContributionFill>
    );
};
SideBySideEditor.defaultProps = { ID: SIDE_BY_SIDE_EDITOR_ID };


export const EditorContribution = props => {
  Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
    id: ResourceEditor.defaultProps.ID,
    name: ResourceEditor.displayName,
  }, [ { __typename: CacheTypenames.ResourceEditorInput }, { __typename: CacheTypenames.UntitledEditorInput } ]);

  Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
    id: TextDiffEditor.defaultProps.ID,
    name: ResourceEditor.displayName,
  }, [ { __typename: CacheTypenames.DiffEditorInput } ]);

  Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
    id: BinaryResourceDiffEditor.defaultProps.ID,
    name: ResourceEditor.displayName,
  }, [ { __typename: CacheTypenames.DiffEditorInput } ]);

  Registry.as<IEditorRegistry>(ClientExtensions.Editors).registerEditor({
    id: SideBySideEditor.defaultProps.ID,
    name: ResourceEditor.displayName,
  }, [ { __typename: CacheTypenames.SideBySideEditorInput } ]);

  return (
    <>
      <ResourceEditor {...props} />
      <TextDiffEditor {...props} />
      <BinaryResourceDiffEditor {...props} />
      <SideBySideEditor {...props} />
    </>
  );
};
